# coding: utf-8
u"""Роутеры для приложений Django."""
from abc import ABCMeta

from django.core.exceptions import ImproperlyConfigured


class ServiceDbRouterBase(object):

    u"""Основа роутера для моделей приложений, использующих сервисную БД.

    Все модели, имена которых указаны в атрибуте ``service_db_model_names``,
    закрепляет за сервисной БД, а остальные модели приложения закрепляет за
    основной БД. Имя приложения указывается в атрибуте ``app_name``.

    Пример использования:

        class DatabaseRouter(ServiceDbRouterBase):
            app_name = 'esia_saml'
            service_db_model_names = set(('EsiaRequestLog',))
    """
    __metaclass__ = ABCMeta

    # имя приложения
    app_name = None

    # имена моделей плагина, которые должны храниться в сервисной БД
    service_db_model_names = None

    def __init__(self):
        u"""Определяет алиасы основной и сервисной баз данных."""
        from django.conf import settings
        from django.db.utils import DEFAULT_DB_ALIAS

        aliases = settings.DATABASES.keys()
        if len(aliases) != 2:
            # Роутер поддерживает только конфигурации с двумя БД.
            raise ImproperlyConfigured(
                u'Database router support only two databases'
            )

        self.default_db_alias = DEFAULT_DB_ALIAS

        alias_index = 1 if aliases[0] == DEFAULT_DB_ALIAS else 0
        self.service_db_alias = aliases[alias_index]

    def db_for_read(self, model, **hints):
        u"""Возвращает имя БД для чтения данных из модели *model*."""
        if model._meta.app_label != self.app_name:
            # модель не имеет отношения к плагину
            return None
        elif model.__name__ in self.service_db_model_names:
            return self.service_db_alias
        else:
            return self.default_db_alias

    def db_for_write(self, model, **hints):
        u"""Возвращает имя БД для записи данных в модель *model*."""
        return self.db_for_read(model, **hints)

    def allow_syncdb(self, db, model):
        u"""Определяет, нужно ли выполнять синхронизацию модели с БД."""
        assert db in (self.default_db_alias, self.service_db_alias)

        if model._meta.app_label == self.app_name:
            return (
                (
                    db == self.default_db_alias and
                    model.__name__ not in self.service_db_model_names
                ) or (
                    db == self.service_db_alias and
                    model.__name__ in self.service_db_model_names
                )
            )

    def allow_migrate(self, db, model):
        u"""Определяет, нужно ли выполнять миграцию для модели в БД."""
        return self.allow_syncdb(db, model)
